#version 330

uniform sampler2D colorTexture;
uniform mat4 modelMatrix;

struct Light {
	vec3 l_position;
	vec3 l_ambient;
	vec3 l_diffuse;
	vec3 l_specular;
};

uniform Material {
	vec4 m_diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	float test;
};

uniform Light lights[10];

in vec3 fragVert;
in vec3 fragNormal;
in vec2 fragTexCoord;

out vec3 fragColor;

void main()
{
	vec4 surfaceColor = texture(colorTexture, fragTexCoord);

	vec3 surfaceToLight = normalize(lights[0].l_position - fragVert);

	// ambient
	vec3 ambient = lights[0].l_ambient * surfaceColor.rgb * vec3(1.0, 0.0, 0.0);

	// diffuse 
	float diffuseCoefficient = max(0.0, dot(fragNormal, surfaceToLight));
	vec3 diffuse = diffuseCoefficient * lights[0].l_diffuse * surfaceColor.rgb;

    //calculate final color of the pixel
    vec3 finalColor = ambient + diffuse;

	fragColor = vec4(finalColor.r, finalColor.g, finalColor.b, 1);
}