#version 330

in vec2 fragTexCoord;

out vec4 color;

uniform sampler2D colorTexture;
uniform sampler2D depthTexture;

uniform mat4 viewProjectionInverseMatrix;
uniform mat4 previousViewProjectionMatrix;

void main(){
	int numSamples = 10;

	// Get the depth buffer value at this pixel.  
    float zOverW = texture2D(depthTexture, fragTexCoord).r;  
   
    // H is the viewport position at this pixel in the range -1 to 1.  
    vec4 H = vec4(fragTexCoord.y * 2 - 1, (1 - fragTexCoord.x) * 2 - 1,  zOverW, 1);  
   
    // Transform by the view-projection inverse.  
    vec4 D = H * viewProjectionInverseMatrix;  
   
    // Divide by w to get the world position.  
    vec4 worldPos = D / D.w; 
    
	// Current viewport position  
    vec4 currentPos = H;  

    // Use the world position, and transform by the previous view-  
    // projection matrix.  
    vec4 previousPos = worldPos * previousViewProjectionMatrix;  

    // Convert to nonhomogeneous points [-1,1] by dividing by w.  
    previousPos /= previousPos.w;  

    // Use this frame's position and last frame's to compute the pixel  
    // velocity.  
    vec2 velocity = ((currentPos - previousPos) / 15).xy;  

	// Get the initial color at this pixel. 
	vec2 currentFragTexCoord = fragTexCoord;

    vec4 colorTmp = texture2D(colorTexture, currentFragTexCoord);  
    currentFragTexCoord += velocity;  
    for(int i = 1; i < numSamples; ++i, currentFragTexCoord += velocity)  
    {  
        // Sample the color buffer along the velocity vector.  
        vec4 currentColor = texture2D(colorTexture, currentFragTexCoord);  
        // Add the current color to our color sum.  
        colorTmp += currentColor;  
    }  
    // Average all of the samples to get the final blur color.  
    color = (colorTmp / numSamples);
}