#version 330                                                                        
                                                                                    
layout(points) in;                                                                   
layout(points, max_vertices = 30) out;                                                      
                                                                                    
in float Type0[];                                                                   
in vec3 Position0[];                                                                
in vec3 Velocity0[];                                                                
in float Age0[];     
in float Lifetime0[];                                                               
in float Transparency0[]; 
                                                                           
out float Type1;                                                                    
out vec3 Position1;                                                                 
out vec3 Velocity1;                                                                 
out float Age1; 
out float Lifetime1;                                                                    
out float Transparency1; 
		                                                                            
uniform float gDeltaTimeMillis;                                                     
uniform float gTime;   
uniform sampler1D gRandomTexture;                                                              
uniform float gShellLifetime;  
uniform vec3 gEmitterPosition;                                                      
                                                                                     
#define PARTICLE_TYPE_NEW 1.0f        
#define PARTICLE_TYPE_SHELL 2.0f       

float rand(vec2 co, float age){
	float t = mod(age, 1000.0);

	t = 1/t;

  return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453 * t);
}

float randomPosition(vec2 v, float age) {
	float random = rand(v, age);

	return (-0.5 + random) / 8;
}

void main() {
	float Age = Age0[0] + gDeltaTimeMillis;                                         
                 
	// new (dead) particle, hide                                   
    if (Type0[0] == PARTICLE_TYPE_NEW) {  
		// time to render new particle
		if (Age0[0] <= gTime) {   
			Type1 = PARTICLE_TYPE_SHELL; 
			float randF = randomPosition(gEmitterPosition.xy, Age0[0]);
			                                            
			Position1 = vec3(gEmitterPosition.x, gEmitterPosition.y + randF, gEmitterPosition.z + randF);                                                   
			Velocity1 = Velocity0[0];                                                   
			Age1 = gTime;      
			Lifetime1 = gShellLifetime;

			// show particle
			Transparency1 = 1.0;  
		}
		else {             
			Type1 = PARTICLE_TYPE_NEW;      
			Position1 = gEmitterPosition;                                                   
			Velocity1 = Velocity0[0];                                                   
			Age1 = Age0[0];      
			Lifetime1 = Lifetime0[0];

			// keep particle hided
			Transparency1 = 0.0;
		}                                                            
    }
	// render normal (alive) particle
	else if (Type0[0] == PARTICLE_TYPE_SHELL) {  
		// kill particle if it had a long enough life 
		if (Age0[0] + Lifetime0[0] + 1000 <= gTime) {   
			Type1 = PARTICLE_TYPE_NEW;   
			Position1 = Position0[0];                                                    
			Velocity1 = Velocity0[0];                                                   
			Age1 = Age0[0] + Lifetime0[0] + 1000;  
			Lifetime1 = Lifetime0[0];

			// hide particle
			Transparency1 = 0.0;      
		}
		// keep showing the particle at its original position (todo: add very little movement)
		else {
			Type1 = PARTICLE_TYPE_SHELL;                                             
			Position1 = Position0[0];                                               
			Velocity1 = Velocity0[0];                                                   
			Age1 = Age0[0];  
			Lifetime1 = Lifetime0[0];

			float diff   = gTime - Age1;

			// time of death is comming closer => reduce transparency _slowly_
			if (diff > (Lifetime0[0] * 0.8)) {
				float remainingLifetime = Lifetime0[0] - (Lifetime0[0] * 0.8);
				diff = Lifetime0[0] - diff;

				float factor = (diff + 4000) / Lifetime0[0];
			
				// reduce opacity when time passes by
				Transparency1 = Transparency0[0] * factor;  
			}                                            
			else {
				Transparency1 = Transparency0[0];  
			}                 
		}
	}

	EmitVertex();                                                               
	EndPrimitive(); 
}